/* parser.mly */                                    
/* @authors: Liming Zhang  @version:0.0 */

%{ 
  open Ast
  let parse_error s = Printf.ksprintf failwith "ERROR: %s" s
%}


%token LPAREN RPAREN LBRACE RBRACE LBRACKET RBRACKET SEMICOLON COMMA COLON DOT
%token IF ELSE FOR WHILE RETURN NEW SWITCH CASE FUNCTION DEFAULT VALUE
%token INT FLOAT STRING VOID BOOLEAN
%token LT GT LE GE NE EQ NOT XOR XNOR AND NAND OR NOR ASSIGN
%token PLUS MINUS MULTIPLY DIVIDE MODULUS
%token RECIPROCAL
%token RES BIT 
%token SERIES PARALLEL
%token <int> INTLiteral
%token <float> FLOATLiteral
%token <string> ID
%token <string> BITLiteral
%token <string> Sentence

%token EOF


%nonassoc NOELSE
%nonassoc ELSE


%right ASSIGN
%left SERIES PARALLEL
%left OR NOR
%left XOR XNOR
%left AND NAND
%left EQ NE
%left GT GE LT LE
%left PLUS MINUS
%left MULTIPLY DIVIDE MODULUS 
%right NOT RECIPROCAL 
%nonassoc LBRACE 
%left DOT

%start program
%type <Ast.program> program

%%

program:
   /* nothing */ { [], [] }
 | program vdecl { ($2 :: fst $1), snd $1 }
 | program fdecl { fst $1, ($2 :: snd $1) }

fdecl:
   type_decl ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
     { { rtype   = $1;
         fname   = $2;
	       formals = $4;
	       locals  = List.rev $7;
	       body    = $8
       }
     }

formals_opt:
      /* nothing */ { [] }
    | formal_list   { List.rev $1 }

formal_list:
      formal                   { [$1] }
    | formal_list COMMA formal { $3 :: $1 }

formal:
    type_decl ID
    { { vtype = $1;
        vname = $2;
    } }

vdecl_list:
      /* nothing */    { [] }
    | vdecl_list vdecl { $2 :: $1 }

vdecl:
    type_decl ID SEMICOLON
    { { vtype = $1;
        vname = $2
      }
    }



type_decl:
    INT     { Int }
  | FLOAT   { Float }
  | STRING  { String }
  | BIT     { Bit }
  | RES     { Res }
  | VOID    { Void }
  | BOOLEAN { Boolean }

      
stmt_list :
    /*nothing*/                   { [] }
    | stmt_list stmt	          { $2 :: $1 }



stmt:
      expr SEMICOLON                               				  { Expr($1) }
    | LBRACE vdecl_list stmt_list RBRACE             { Block(List.rev $2, List.rev $3) }
    | RETURN expr SEMICOLON                          { Return($2) }
    | IF LPAREN expr RPAREN stmt %prec NOELSE        { If($3, $5, Block([],[])) }
    | IF LPAREN expr RPAREN stmt ELSE stmt           { If($3, $5, $7) }
    | WHILE LPAREN expr RPAREN stmt                  { While($3, $5) }
    | FOR LPAREN expr SEMICOLON expr SEMICOLON expr RPAREN stmt         { For($3, $5, $7, $9) }


expr:
      literal              { Literal($1) }

    | expr PLUS expr       { Binop($1, Add, $3) }
    | expr MINUS expr      { Binop($1, Sub, $3) }
    | expr MULTIPLY expr      { Binop($1, Mult, $3) }
    | expr DIVIDE expr     { Binop($1, Div, $3) }
     | expr GT expr         { Binop($1, Gt, $3) }
		|  expr NE expr         { Binop($1, Ne, $3) }
		|  expr LT expr         { Binop($1, Lt, $3) }
		|   expr LE expr         { Binop($1, Le, $3) }
		| expr GE expr         { Binop($1, Ge, $3) }
		| expr EQ expr         { Binop($1, Eq, $3) }
   
    | expr SERIES expr     { Binop($1, Series, $3)    }
    | expr PARALLEL expr   { Binop($1, Parallel, $3)  }
		| expr AND expr     { Binop($1, And, $3)    }
    | expr OR expr         { Binop($1, Or, $3)   }

    | RECIPROCAL expr      { Unary(Reciprocal, $2)}
    
    | PLUS expr            { Unary(Plus, $2) }
    | MINUS expr           { Unary(Minus, $2) }

   
    | lvalue                              { $1 }
    | lvalue ASSIGN expr                  { Assign($1, $3) }

    | LPAREN expr RPAREN                  { $2 }
    | ID LPAREN arg_list RPAREN           { Call($1, List.rev $3) }
    

lvalue:                                                           
    ID                                    { Id($1) }       
   | expr DOT VALUE                          { Afterop($1, Dot)}
  

literal:
    INTLiteral                                   { IntLit($1) }
    | FLOATLiteral                               { FloatLit($1) }
    | BITLiteral                                 { BinaryLit($1) }
   	| Sentence                                   {SentenceLit($1)}

arg_list:
     /* nothing */                        { [] }
    | expr                                { [$1] }
    | arg_list COMMA expr                 { $3::$1 }
